/*
 * PmodGPS.h
 *
 *  Created on: Nov 20, 2017
 *      Author: Krzysztof Chojnowski
 */

#ifndef PMODGPS_H_
#define PMODGPS_H_

#include <stdbool.h>
#include "stm32l4xx_hal.h"

/**
 * @brief Configure ARDUINO CONNECTOR pins. This configuration uses two transmission pins (RX, TX),
 *        3DF pin for state indication, 1PPS pin for time synchronization.
 *        PD6  -> 3DF
 *        PC4  -> RX
 *        PC5  -> TX
 *        PD10 -> 1PPS
 */
void PmodGPS_Config(void);

/**
 * @brief Enables clocks required by USART3 and GPIO, initializes USART3 GPIO pins in alternate mode
 *        and enables USART interrupt. This function is required by HAL library and needs to be
 *        exported because two USART ports are used in this example.
 */
void PmodGPS_HAL_UART_MspInit(UART_HandleTypeDef *huart);

/**
 * @brief Sends data to USART3 port HAL driver in non-blocking mode.
 */
void PmodGPS_Write(char* data, int len);

/**
 * @brief Enables a non-blocking read operation from the USART3 port. If the interrupt handling routine
 *        is defined, it will be called after given number of bytes is received. In this example
 *        the USART3 interrupt routines are defined in the main.c file.
 */
void PmodGPS_Read(uint8_t* data, int len);

#endif /* PMODGPS_H_ */
