/*
******************************************************************************
File:     main.c
Info:     Generated by Atollic TrueSTUDIO(R) 8.0.0   2017-10-28

The MIT License (MIT)
Copyright (c) 2009-2017 Atollic AB

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

******************************************************************************
*/

#include "stm32l4xx.h"
#include "stm32l4xx_hal.h"
#include "system_clock.h"
#include "serial.h"
#include "led.h"

#include "PmodGPS.h"

// Number of data buffers shared between interrupts and application contexts.
#define DATA_BUFFERS_COUNT 2

typedef struct {
	uint8_t buffer[128];
	uint32_t index;
} DataBuffer;

// Data buffers used for data reception.
DataBuffer dataBuffers[DATA_BUFFERS_COUNT];

// The index of currently written buffer.
uint32_t currentDataBuffer = 0;

// Data ready flag.
uint8_t dataReady = 0;

int main(void)
{
  HAL_Init();
  SystemClock_Config();
  Led_Config();
  Serial_Config();

  // Clear all buffers.
  for(uint32_t i = 0; i < DATA_BUFFERS_COUNT; i++)
    dataBuffers[i].index = 0;

  PmodGPS_Config();

  // Send the firmware version query to the PmodGPS.
  PmodGPS_Write("$PMTK605*31\r\n", 13);

  // Start receiving data to the first buffer.
  PmodGPS_Read(dataBuffers[0].buffer, 1);

  // This loop processes the received data by sending the full messages over the serial port.
  // In this simple implementation the processing of a buffer has to end before the next
  // message is received.
  while(1)
  {
    // Wait for the data to be ready.
    while(dataReady == 0)
      __NOP();

    dataReady = 0;

    // The buffer before the current one contains the data received from PmodGPS.
    uint32_t readyDataBuffer =
      (currentDataBuffer > 0) ? (currentDataBuffer - 1) : (DATA_BUFFERS_COUNT - 1);

    // Write the data from the buffer to the serial port and clear the buffer.
    Serial_Write((char*)dataBuffers[readyDataBuffer].buffer, dataBuffers[readyDataBuffer].index + 1);
    dataBuffers[readyDataBuffer].index = 0;
  }
}

/**
 * @brief General UART configuration function called by the HAL library during each port
 *        initialization. This function calls the MspInit function of the appropriate driver.
 */
void HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
  if(huart->Instance == USART3)
    PmodGPS_HAL_UART_MspInit(huart);
  else if(huart->Instance == LPUART1)
    Serial_HAL_UART_MspInit(huart);
}
/**
 * @brief Data reception interrupt handler. The interrupt is configured by the PmodGPS
 *        driver (USART3).
 */
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
  // Check if the whole message has already been received.
  if(dataBuffers[currentDataBuffer].buffer[dataBuffers[currentDataBuffer].index] == '\n') {
    // Notify the application about incoming data
    // and take the next buffer for further data reception.
    dataReady = 1;

    currentDataBuffer++;
    if(currentDataBuffer == DATA_BUFFERS_COUNT)
      currentDataBuffer = 0;
  } else {
    dataBuffers[currentDataBuffer].index++;
  }

  // Receive another byte to the current buffer.
  PmodGPS_Read(&dataBuffers[currentDataBuffer].buffer[dataBuffers[currentDataBuffer].index], 1);
}

/**
 * @brief GPIO interrupt handler. The interrupt is configured by the PmodGPS driver and shows the state
 *        of the 3DF and 1PPS lines using two LEDs.
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  if(GPIO_Pin == GPIO_PIN_6) {
    if(HAL_GPIO_ReadPin(GPIOD, GPIO_PIN_6) == GPIO_PIN_SET)
      Led_TurnOn(LED0);
    else
      Led_TurnOff(LED0);
  }

  if(GPIO_Pin == GPIO_PIN_10) {
    if(HAL_GPIO_ReadPin(GPIOD, GPIO_PIN_10) == GPIO_PIN_SET)
      Led_TurnOn(LED1);
    else
      Led_TurnOff(LED1);
  }
}
